@@ -104,7 +104,7 @@ use core::marker::PhantomData;
 
 use crate::gpio::dynpin::DynFunction;
 #[cfg(feature = "eh1_0_alpha")]
-use eh1_0_alpha::digital::blocking as eh1;
+use eh1_0_alpha::digital as eh1;
 use hal::digital::v2::{InputPin, OutputPin, StatefulOutputPin, ToggleableOutputPin};
 
 use core::mem::transmute;
@@ -845,12 +845,20 @@ where
 }
 
 #[cfg(feature = "eh1_0_alpha")]
-impl<I, C> eh1::OutputPin for Pin<I, Output<C>>
+impl<I, C> eh1::ErrorType for Pin<I, Output<C>>
 where
     I: PinId,
     C: OutputConfig,
 {
     type Error = Infallible;
+}
+
+#[cfg(feature = "eh1_0_alpha")]
+impl<I, C> eh1::blocking::OutputPin for Pin<I, Output<C>>
+where
+    I: PinId,
+    C: OutputConfig,
+{
     #[inline]
     fn set_high(&mut self) -> Result<(), Self::Error> {
         self._set_high();
@@ -864,11 +872,10 @@ where
 }
 
 #[cfg(feature = "eh1_0_alpha")]
-impl<I> eh1::InputPin for Pin<I, ReadableOutput>
+impl<I> eh1::blocking::InputPin for Pin<I, ReadableOutput>
 where
     I: PinId,
 {
-    type Error = Infallible;
     #[inline]
     fn is_high(&self) -> Result<bool, Self::Error> {
         Ok(self._is_high())
@@ -880,12 +887,20 @@ where
 }
 
 #[cfg(feature = "eh1_0_alpha")]
-impl<I, C> eh1::InputPin for Pin<I, Input<C>>
+impl<I, C> eh1::ErrorType for Pin<I, Input<C>>
 where
     I: PinId,
     C: InputConfig,
 {
     type Error = Infallible;
+}
+
+#[cfg(feature = "eh1_0_alpha")]
+impl<I, C> eh1::blocking::InputPin for Pin<I, Input<C>>
+where
+    I: PinId,
+    C: InputConfig,
+{
     #[inline]
     fn is_high(&self) -> Result<bool, Self::Error> {
         Ok(self._is_high())
@@ -897,12 +912,11 @@ where
 }
 
 #[cfg(feature = "eh1_0_alpha")]
-impl<I, C> eh1::ToggleableOutputPin for Pin<I, Output<C>>
+impl<I, C> eh1::blocking::ToggleableOutputPin for Pin<I, Output<C>>
 where
     I: PinId,
     C: OutputConfig,
 {
-    type Error = Infallible;
     #[inline]
     fn toggle(&mut self) -> Result<(), Self::Error> {
         self._toggle();
@@ -911,7 +925,7 @@ where
 }
 
 #[cfg(feature = "eh1_0_alpha")]
-impl<I, C> eh1::StatefulOutputPin for Pin<I, Output<C>>
+impl<I, C> eh1::blocking::StatefulOutputPin for Pin<I, Output<C>>
 where
     I: PinId,
     C: OutputConfig,
@@ -1088,6 +1102,33 @@ macro_rules! bsp_pins {
             ///
             /// This type is intended to provide more meaningful names for the
             /// given pins.
+            ///
+            /// To enable specific functions of the pins you can use the
+            /// [rp2040_hal::gpio::pin::Pin::into_mode] function with
+            /// one of:
+            /// - [rp2040_hal::gpio::pin::FunctionI2C]
+            /// - [rp2040_hal::gpio::pin::FunctionPwm]
+            /// - [rp2040_hal::gpio::pin::FunctionSpi]
+            /// - [rp2040_hal::gpio::pin::FunctionXip]
+            /// - [rp2040_hal::gpio::pin::FunctionPio0]
+            /// - [rp2040_hal::gpio::pin::FunctionPio1]
+            /// - [rp2040_hal::gpio::pin::FunctionUart]
+            ///
+            /// like this:
+            ///```no_run
+            /// use rp2040_hal::{pac, gpio::{bank0::Gpio12, Pin, Pins, PushPullOutput}, sio::Sio};
+            ///
+            /// let mut peripherals = pac::Peripherals::take().unwrap();
+            /// let sio = Sio::new(peripherals.SIO);
+            /// let pins = Pins::new(peripherals.IO_BANK0,peripherals.PADS_BANK0,sio.gpio_bank0, &mut peripherals.RESETS);
+            ///
+            /// let _spi_sclk = pins.gpio2.into_mode::<rp2040_hal::gpio::FunctionSpi>();
+            /// let _spi_mosi = pins.gpio3.into_mode::<rp2040_hal::gpio::FunctionSpi>();
+            /// let _spi_miso = pins.gpio4.into_mode::<rp2040_hal::gpio::FunctionSpi>();
+            ///```
+            ///
+            /// **See also [rp2040_hal::gpio::pin] for more in depth information
+            /// about this**!
             pub struct Pins {
                 $(
                     $( #[$id_cfg] )*
