@@ -15,11 +15,15 @@ pub use paste;
 
 pub extern crate rp2040_pac as pac;
 
+#[macro_use]
+mod intrinsics;
+
 pub mod adc;
 pub(crate) mod atomic_register_access;
 pub mod clocks;
 mod critical_section_impl;
 pub mod dma;
+mod float;
 pub mod gpio;
 pub mod i2c;
 pub mod multicore;
