@@ -11,7 +11,7 @@ const PIO_INSTRUCTION_COUNT: usize = 32;
 
 /// PIO Instance
 pub trait PIOExt:
-    core::ops::Deref<Target = rp2040_pac::pio0::RegisterBlock> + SubsystemReset + Sized
+    core::ops::Deref<Target = rp2040_pac::pio0::RegisterBlock> + SubsystemReset + Sized + Send
 {
     /// Create a new PIO wrapper and split the state machines into individual objects.
     #[allow(clippy::type_complexity)] // Required for symmetry with PIO::free().
@@ -94,7 +94,7 @@ impl<P: PIOExt> core::fmt::Debug for PIO<P> {
 
 // Safety: `PIO` only provides access to those registers which are not directly used by
 // `StateMachine`.
-unsafe impl<P: PIOExt + Send> Send for PIO<P> {}
+unsafe impl<P: PIOExt> Send for PIO<P> {}
 
 // Safety: `PIO` is marked Send so ensure all accesses remain atomic and no new concurrent accesses
 // are added.
@@ -244,13 +244,11 @@ impl<P: PIOExt> PIO<P> {
 /// let mut peripherals = pac::Peripherals::take().unwrap();
 /// let (mut pio, sm0, _, _, _) = peripherals.PIO0.split(&mut peripherals.RESETS);
 /// // Install a program in instruction memory.
-/// let program = pio_proc::pio!(
-///     32,
-///     ".wrap_target
-///     set pins, 1 [31]
-///     set pins, 0 [31]
-/// .wrap
-///     "
+/// let program = pio_proc::pio_asm!(
+///     ".wrap_target",
+///     "set pins, 1 [31]",
+///     "set pins, 0 [31]",
+///     ".wrap"
 /// ).program;
 /// let installed = pio.install(&program).unwrap();
 /// // Configure a state machine to use the program.
@@ -315,7 +313,7 @@ impl<P: PIOExt> InstalledProgram<P> {
 }
 
 /// State machine identifier (without a specified PIO block).
-pub trait StateMachineIndex {
+pub trait StateMachineIndex: Send {
     /// Numerical index of the state machine (0 to 3).
     fn id() -> usize;
 }
@@ -559,6 +557,22 @@ impl<SM: ValidStateMachine> StateMachine<SM, Stopped> {
         }
     }
 
+    /// Change the clock divider of a stopped state machine.
+    pub fn set_clock_divisor(&mut self, divisor: f32) {
+        // sm frequency = clock freq / (CLKDIV_INT + CLKDIV_FRAC / 256)
+        let int = divisor as u16;
+        let frac = ((divisor - int as f32) * 256.0) as u8;
+
+        self.sm.sm().sm_clkdiv.write(|w| {
+            unsafe {
+                w.int().bits(int);
+                w.frac().bits(frac);
+            }
+
+            w
+        });
+    }
+
     /// Sets the pin state for the specified pins.
     ///
     /// The user has to make sure that they do not select any pins that are in use by any
@@ -902,7 +916,7 @@ pub struct Interrupt<P: PIOExt> {
 }
 
 // Safety: `Interrupt` provides exclusive access to interrupt registers.
-unsafe impl<P: PIOExt + Send> Send for Interrupt<P> {}
+unsafe impl<P: PIOExt> Send for Interrupt<P> {}
 
 // Safety: `Interrupt` is marked Send so ensure all accesses remain atomic and no new concurrent
 // accesses are added.
