@@ -67,7 +67,7 @@
 //! pwm.set_div_int(1u8); // To set integer part of clock divider
 //! pwm.set_div_frac(0u8); // To set fractional part of clock divider
 //!
-//! pwm.get_top(); // To set the TOP register
+//! pwm.get_top(); // To get the TOP register
 //! pwm.set_top(u16::MAX); // To set the TOP register
 //!
 //! ```
@@ -87,8 +87,6 @@ use crate::{
     resets::SubsystemReset,
     typelevel::Sealed,
 };
-#[cfg(feature = "eh1_0_alpha")]
-use eh1_0_alpha::pwm::blocking as eh1;
 use embedded_hal::PwmPin;
 use pac::PWM;
 
@@ -616,38 +614,6 @@ impl<S: SliceId, M: SliceMode> PwmPin for Channel<S, M, A> {
     }
 }
 
-#[cfg(feature = "eh1_0_alpha")]
-impl<S: SliceId, M: SliceMode> eh1::PwmPin for Channel<S, M, A> {
-    type Duty = u16;
-    type Error = core::convert::Infallible;
-
-    /// We cant disable the channel without disturbing the other channel.
-    /// So this just sets the duty cycle to zero
-    fn disable(&mut self) -> Result<(), Self::Error> {
-        self.duty_cycle = self.regs.read_cc_a();
-        self.regs.write_cc_a(0);
-        Ok(())
-    }
-
-    fn enable(&mut self) -> Result<(), Self::Error> {
-        self.regs.write_cc_a(self.duty_cycle);
-        Ok(())
-    }
-
-    fn get_duty(&self) -> Result<Self::Duty, Self::Error> {
-        Ok(self.regs.read_cc_a())
-    }
-
-    fn get_max_duty(&self) -> Result<Self::Duty, Self::Error> {
-        Ok(self.regs.read_top())
-    }
-
-    fn set_duty(&mut self, duty: Self::Duty) -> Result<(), Self::Error> {
-        self.regs.write_cc_a(duty);
-        Ok(())
-    }
-}
-
 impl<S: SliceId, M: SliceMode> PwmPin for Channel<S, M, B> {
     type Duty = u16;
 
@@ -674,37 +640,6 @@ impl<S: SliceId, M: SliceMode> PwmPin for Channel<S, M, B> {
         self.regs.write_cc_b(duty)
     }
 }
-#[cfg(feature = "eh1_0_alpha")]
-impl<S: SliceId, M: SliceMode> eh1::PwmPin for Channel<S, M, B> {
-    type Duty = u16;
-    type Error = core::convert::Infallible;
-
-    /// We cant disable the channel without disturbing the other channel.
-    /// So this just sets the duty cycle to zero
-    fn disable(&mut self) -> Result<(), Self::Error> {
-        self.duty_cycle = self.regs.read_cc_b();
-        self.regs.write_cc_b(0);
-        Ok(())
-    }
-
-    fn enable(&mut self) -> Result<(), Self::Error> {
-        self.regs.write_cc_b(self.duty_cycle);
-        Ok(())
-    }
-
-    fn get_duty(&self) -> Result<Self::Duty, Self::Error> {
-        Ok(self.regs.read_cc_b())
-    }
-
-    fn get_max_duty(&self) -> Result<Self::Duty, Self::Error> {
-        Ok(self.regs.read_top())
-    }
-
-    fn set_duty(&mut self, duty: Self::Duty) -> Result<(), Self::Error> {
-        self.regs.write_cc_b(duty);
-        Ok(())
-    }
-}
 
 impl<S: SliceId, M: SliceMode + ValidSliceMode<S>> Channel<S, M, A> {
     /// Capture a gpio pin and use it as pwm output for channel A
